+++
date = '2017-01-01'
title = 'Neverwinter Nights 2 related dev'
tags = ["NWN2", "reverse-eng", "3d", "Dlang", "ansible", "virtualization", "webdev", "devops"]
+++


Being a Neverwinter Nights 2 server admin and developer, I've got a lot of good oportunities to experiment with various technologies.

Read more to get more information on:
- **nwn-lib-d**: a library and tools for parsing NWN2 binary files
- **lcda-account-manager**: A frontend and backend for browsing NWN2 characters and managing account
- **ansible-nwn2server**: an ansible playbook to setup a NWN2 server with a virtualized windows machine for the NWN2Server process

<!--more-->

## Library for parsing game engine files (nwn-lib-d)

- Parsing and serialization of GFF files, a general purpose, treelike binary storage format.
	+ __Profiling and optimizations__. There are currently two versions of the parser: one read/write that may be slow as it instantiates all data structures in memory, and the second one which is read only, very fast and memory efficient as it only crawls over the file without allocating memory. Some work is in progress to make the second read/write.
	+ Keep the structure of the stored data as is (other existing parsers reorder elements). The checksum of the binary file should not be modified if the file is not changed between parsing and serialization.
- __Reverse engineered__ [TRN.ASWM data](https://gist.github.com/CromFr/104bac52dc9191099d9d9e3dbd2c4975), a structure containing compressed 3D mesh with pathing information (pathfinding pre-calculated data, foot step sounds etc.)
- Other NWN2 simple data structures like TLK (translation tables), ERF (tar-like archive), and FoxproDB (used by NWN2 as a persistent key/value database)

#### Skills
- D language
- __Reverse engineering__ binary files
- __3D model generation for visualizing__ map information (shapes, face junctions, face properties, pathfinding data, ...)
- Pathfinding and 3D mesh-cutting and merge __algorithms__
- __Profiling__, __optimizations__
- Heavy __unit testing__, __coverage analysis__

#### More
- Source code: https://github.com/CromFr/nwn-lib-d
- Status: <img class="raw" src="https://github.com/CromFr/nwn-lib-d/workflows/nwn-lib-d/badge.svg"/>
[<img class="raw" src="https://codecov.io/gh/CromFr/nwn-lib-d/branch/master/graph/badge.svg"/>](https://codecov.io/gh/CromFr/nwn-lib-d)
- Reversed spec for TRN.ASWM format: https://gist.github.com/CromFr/104bac52dc9191099d9d9e3dbd2c4975

## Account manager

{{< cinema path="/images/posts/lcdaaccounts-*.*" >}}

It's a big project aiming to provide to players a __web interface and an open API__ for managing their server account and browsing characters.
It makes heavy use of my _nwn-lib-d_ library to extract information distributed across ~300MB of various game files as fast as possible.

The backend is written in D, using the Vibe.d web framework and nwn-lib-d parsing library (the main reason being that D is a good prototyping language, and is fun to write :) ).

The frontend is written in Typescript using Angular2+, MaterializeCSS (JS, JQuery, SCSS)

- Provides a way to access player's
	+ Characters
		* Basic information
		* Quest status
		* Dungeon completion
		* ...
		* Public status to share characters infos with other players
		* Custom notes
	+ Account settings
		* Authorized application (token-based)
		* Password change
- __Authentication portal__, similar to OAuth (OAuth2 compatibility is a milestone)
- Cache system & rate limiting to prevent from DOSing the service.
- Localization using the NWN2 translation files (TLK).

#### Skills
- D language, with Vibe.d web library
- Frontend design, with Angular / MaterializeCSS
- plus the skills detailed in nwn-lib-d

#### More
- Source code: [Frontend](https://github.com/CromFr/LcdaAccounts) [Backend](https://github.com/CromFr/LcdaApi)
- Production frontend: https://account.lcda-nwn2.fr (French)
	+ Character example: https://account.lcda-nwn2.fr/Crom%2029/vault/krogar (French)
- Production API: https://api.lcda-nwn2.fr/


## Server infrastructure

The previous server was hosted on a Windows XP machine, behind a home connection.

Since the OS was aging (understand _utterly out of date_), and most software do not run anymore on the platform (and the lack of decent remote control / automation), I decided to setup a brand new server running __GNU/Linux__, with a __light windows virtual machine__ just to run the game server (the game server only runs only under Windows, and require DLL injection support for plugins, which puts Wine out of the game).

### General
- Dedicated server hosted on Kimsufi (OVH) servers (low budget / second hand servers)
- Ubuntu 17.04 (LTS) host system (when created)
	+ MariaDB used by nwn2server
	+ LcdaApi and LcdaAccounts services
	+ Automated nwn2server updates
	+ Automated player data & server __backup__
	+ Other web services like a [Wiki](https://wiki.lcda-nwn2.fr/), LcdaAccounts, LcdaApi, ...
- __KVM Virtualization__
	+ Running only the nwn2server process, on Windows Server 2012
	+ __VirtIO__ for fast network and disk access
	+ __SPICE remote control__ only accessible through a SSH tunnel
- __Automated setup__ using Ansible
- Automated game server updates
	+ Manually updating the game server files is very cumbersome and error-prone
	+ Now all tasks are automated and updating the server is a __"one click" process__

### Ansible

Since Kimsufi services do not provide disk redundancy nor KVM (Keyboard/Video/Mouse) access to the machine (they are the cheapest dedicated servers available), the setup is fully automated using Ansible scripts.

In case of a disk failure, the server can be reinstalled in a __few hours__ (mostly spent on Windows guest setup & updating), and backed up data can be restored quickly.

#### More
- Source code: https://github.com/CromFr/ansible-nwn2server (Note: LCDA-specific scripts are kept in a private repository)

#### Skills
- Server administration
- __Ansible__ automation
- __Optimized virtualization__, __Nested KVM debugging__
- Windows server administration :( (We needed DLL injection support for the nwn2server, which isn't supported by either Wine or ReactOS)
